def is_in_save(current_obj):
    return _get_save_caches(current_obj) is not None


def get_from_cache_during_save(current_obj, caches_key_type, caches_key_obj, fill_cache_func, target_key):
    save_caches = _get_save_caches(current_obj)
    assert save_caches is not None, 'Must only call this during a save operation!'
    cache_key = (caches_key_type, id(caches_key_obj))
    cache = save_caches.get(cache_key)
    if cache is None:
        cache = {}
        fill_cache_func(cache)
        save_caches[cache_key] = cache
    return cache.get(target_key)


def _get_save_caches(current_obj):
    parent = current_obj._parent
    while parent is not None:
        current_obj = parent
        parent = current_obj._parent
    return getattr(current_obj, '_save_caches', None)



from pbxproj import PBXGenericObject


class PBXContainerItemProxy(PBXGenericObject):
    @classmethod
    def create(cls, file_ref, remote_ref, proxy_type=2):
        return cls().parse({
            u'_id': cls._generate_id(),
            u'isa': cls.__name__,
            u'containerPortal': file_ref.get_id(),
            u'proxyType': proxy_type,
            u'remoteGlobalIDString': remote_ref.productReference,
            u'remoteInfo': remote_ref.productName
        })

    def _get_comment(self):
        return u'PBXContainerItemProxy'



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()


import os
from pbxproj.XcodeProject import XcodeProject


def open_project(args):
    if os.path.isdir(args[u'<project>']):
        args[u'<project>'] += u"/project.pbxproj"

    if not os.path.isfile(args[u'<project>']):
        raise Exception(u'Project file not found')

    return XcodeProject.load(args[u'<project>'])


def backup_project(project, args):
    if args[u'--backup']:
        return project.backup()
    return None


def resolve_backup(project, backup_file, args):
    # remove backup if everything was ok.
    if args[u'--backup'] and backup_file:
        os.remove(backup_file)


def command_parser(command, auto_save=True):
    def parser(args):
        try:
            project = open_project(args)
            backup_file = backup_project(project, args)
            print(command(project, args))
            if auto_save:
                project.save()
            resolve_backup(project, backup_file, args)
        except Exception as ex:
            print(u"{0}".format(ex))
            exit(1)
    return parser



import os
from pbxproj import PBXGenericObject


class PBXFileReference(PBXGenericObject):
    @classmethod
    def create(cls, path, tree=u'SOURCE_ROOT'):
        return cls().parse({
            u'_id': cls._generate_id(),
            u'isa': cls.__name__,
            u'path': path,
            u'name': os.path.split(path)[1],
            u'sourceTree': tree
        })

    def set_explicit_file_type(self, file_type):
        if u'lastKnownFileType' in self:
            del self[u'lastKnownFileType']
        self[u'explicitFileType'] = file_type

    def set_last_known_file_type(self, file_type):
        if u'explicitFileType' in self:
            del self[u'explicitFileType']
        self[u'lastKnownFileType'] = file_type

    def get_file_type(self):
        if u'explicitFileType' in self:
            return self.explicitFileType
        return self.lastKnownFileType

    def _print_object(self, indentation_depth=u'', entry_separator=u'\n', object_start=u'\n',
                      indentation_increment=u'\t'):
        return super(PBXFileReference, self)._print_object(u'', entry_separator=u' ', object_start=u'',
                                                           indentation_increment=u'')

    def get_name(self):
        if hasattr(self, u'name'):
            return self.name
        if hasattr(self, u'path'):
            return self.path
        return None

    def remove(self, recursive=True):
        # search on the BuildFiles if there is a build file to be removed, and remove it
        # search for each phase that has a reference to the build file and remove it from it.
        # remove the file reference from it's parent
        pass

